<?php

/**
 * Fonctions pour l'automatisation NextCloud
 * @Projet SambaEdu
 * @Auteurs Equipe Sambaedu
 * @Licence Distribue sous la licence GPL
 * @Repertoire: includes/
 * file: cloud.inc.php
 *
 * Accès minimal à l'API NC pour permettre la création de partages (classes, groupes) et la récupération de la clé client pour le montage RClone
 * 
 * Utilisation de lib php webdav
 * 
 * Seuls les appels utiles sont implémentés.
 * 
 * Authentification
 * 
 * - login avec un compte et sa clé d'api
 * - récupération de la clé api pour un compte utilisateur en passant le mot de passe, ou via un compte admin si c'est possible
 * 
 * Dossiers (api webdav)
 * 
 * - liste des dossiers
 * - mkdir
 * - rm
 * - mv
 * 
 * Partages (api shares)
 * 
 * - liste de droits
 * - ajout d'un droit
 * - suppression d'un droit
 * 
 * 
 * les données de conf sont dans /etc/sambaedu/sambaedu.conf.d/cloud.conf
 * 
 * cloud_name = "nuage"
 * cloud_type = "webdav"
 * cloud_uri = "https://nuage.etab.fr/"
 *
 * si les utilisateurs sont identifiés par leur GUID (et pas leur CN) dans NC
 *
 * cloud_userid = "guid" 
 *
 * un compte admin sur le NC doit être défini dans la conf pour les opérations.
 *
 * cloud_api_user = "nomdeladmincloud"
 * cloud_api_password = "token"
 *
 *
 * la lib sabre/dav est utilisée pour faire les requêtes webdav
 *
 * les partages sont gérés par un utilisateur unique par établissement.
 */
use GuzzleHttp\Exception\GuzzleException;

function cloud_path($config, $user)
{
    switch (@$config['cloud_type']) {
        case "seafile":
            $path = $config['cloud_uri'];
            break;
        default:
            $path = $config['cloud_uri'] . "remote.php/dav/files/" . $user . "/";
    }
    return $path;
}

function get_nc_client_code($config, $user, $password)
{
    $info = false;
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('GET', 'ocs/v2.php/core/getapppassword', [
            'auth' => [
                $user,
                $password
            ],
            'query' => [
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                $info['code'] = $res['ocs']['data']['apppassword'];
                $response = $client->request('GET', 'ocs/v2.php/cloud/user', [
                    'auth' => [
                        $user,
                        $info['code']
                    ],
                    'query' => [
                        'format' => 'json'
                    ]
                ]);
                if ($response) {
                    $res = json_decode((string) $response->getBody(), true);
                    if ($res['ocs']['meta']['status'] == "ok") {
                        $info['id'] = $res['ocs']['data']['id'];
                        // $info['storageLocation'] = $res['ocs']['data']['storageLocation'];
                    }
                }
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException $e) {
        return false;
    }
    return $info;
}

function test_cloud_login($config, $user = "", $password = "")
{
    $data = false;
    if (empty($user) && empty($password)) {
        $creds = [
            $config['cloud_api_user'],
            $config['cloud_api_password']
        ];
    } else {
        $creds = [
            $user,
            $password
        ];
    }

    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ],
        'query' => [
            'format' => 'json'
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('GET', 'ocs/v2.php/cloud/user?format=json', [
            'auth' => $creds
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                $data = $res['ocs']['data'];
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException $e) {
        return $e;
    }
    return $data;
}

function configure_apps_client($config, $user, &$cloud = [])
{
    $data = unserialize($_POST['data'] ?? NULL);
    $nuage = $_POST['nuage'] ?? "";
    $init = $_POST['init'] ?? "0";

    $conf = get_rclone_config($config, $user['cn']);
    if (isset($conf['nuage.apps']) && $init == 0) {
        $config['cloud_uri'] = parse_url($conf['nuage.apps']['url'], PHP_URL_SCHEME) . "://" . parse_url($conf['nuage.apps']['url'], PHP_URL_HOST) . "/";
        // $config['cloud_uri'] = "https://nuage.apps.education.fr/";
        $config['cloud_type'] = $conf['nuage.apps']['type'];
        $config['cloud_vendor'] = $conf['nuage.apps']['vendor'];
        $config['cloud_name'] = 'nuage.apps';
        $config['cloud_api_password'] = $conf['nuage.apps']['pass'];
        $config['cloud_api_user'] = $conf['nuage.apps']['user'];
        $html = '<p>Vous êtes connecté sur <a href="' . $config['cloud_uri'] . '">' . $config['cloud_uri'] . '</a>  avec le compte
<b>' . $config['cloud_api_user'] . '</b>.<br>
Vous trouverez sur votre bureau une icône <b>Mes documents en ligne</b> et en l\'ouvrant un dossier un<b>nuages\\nuage.apps</b> vous donnant accès à vos données.<p>';
        $html .= 'Vous pouvez réinitialiser cette connexion en cas de problème<br>
Pour plus d&lsquo;explications sur la connexion de votre ordinateur personnel à ce cloud : <a href="https://magistere.education.fr/f709">Formation à l&lsquo;usage du cloud</a>
<form name="cloud" action="" method="post">
<input type=hidden name=nuage value="">
<input type=hidden name=init value="1">
<input type=submit name="valider" value="Réinitialiser">
</form>';
        $cloud[] = "nuage.apps";
    } else {
        $config['cloud_uri'] = "https://" . $nuage . ".apps.education.fr/";
        // $config['cloud_uri'] = "https://nuage.apps.education.fr/";
        $config['cloud_type'] = "webdav";
        $config['cloud_vendor'] = "nextcloud";
        $config['cloud_name'] = "nuage.apps";

        require_once (dirname(__FILE__) . '/../vendor/autoload.php');

        $opt = [
            'base_uri' => $config['cloud_uri'],
            'cookies' => true,
            'headers' => [
                'OCS-APIRequest' => "true",
                'User-Agent' => "SambaEdu"
            ],
            'http_errors' => false
        ];
        curl_proxy_options($config, $opt);
        $html = '<form name="cloud" action="" method="post">';
        if (! $data) {
            if (empty($nuage)) {
                // étape 1 : choix du serveur
                $html .= "<p>Pour activer la connexion automatique avec votre espace personnel sur apps.education.fr,<br>
Vous devez au préalable avoir créé votre compte sur apps.education.fr, et activé l&lsquo;application \"nuage\".<br>
Choisissez le serveur auquel vous êtes connectés :<br>
<select name=nuage onchange=\"this.form.submit()\">
    <option value=''>Choisir votre serveur</option>
    <option value='nuage01'>nuage01</option>
    <option value='nuage02'>nuage02</option>
    <option value='nuage03'>nuage03</option>
    <option value='nuage04'>nuage04</option>
    <option value='nuage05'>nuage05</option>
    <option value='nuage06'>nuage06</option>
    <option value='nuage07'>nuage07</option>
    <option value='nuage08'>nuage08</option>
    <option value='nuage09'>nuage09</option>
</select>";
            } elseif (empty($data['poll']['token'])) {
                // Etape 2 : récup de l'url de la page de login et du token associé
                $client = new GuzzleHttp\Client($opt);
                try {
                    $response = $client->request('POST', 'index.php/login/v2', []);
                    if ($response) {
                        // on ouvre la page de login
                        $data = json_decode((string) $response->getBody(), true);
                        $url = $data['login'];
                        $html .= "<input type=hidden name=data value=" . serialize($data) . ">
<p>Pour activer la connexion automatique avec votre espace personnel sur apps.education.fr, Entrez vos identifiants dans la fenêtre qui va apparaître.<br>
Vous devez au préalable avoir créé votre compte sur apps.education.fr, et activé l&lsquo;application \"nuage\".<br>
Le mot de passe saisi sert juste à la vérification et ne sera pas enregistré<p>
<script type=\"text/javascript\">
    function openOnce(url, target){
      var winref = window.open('', target, 'resizable=yes,height=500,width=500');
      if(winref.location.href === 'about:blank'){
         winref.location.href = url;
         var refresh = true;
      }else if(winref.location.href === '" . $url . "/#/'){
         var refresh = true;
      }else{
         var refresh = false;
      }
      return winref;
    }
    openOnce('" . $url . "', 'apps.education');
    var wait = setTimeout(document.cloud.submit(), 29000);
</script>
<input type=hidden name=data value=" . serialize($data) . ">
<input type=hidden name=nuage value=" . $nuage . ">
<input type=\"submit\" name=\"valider\" value=\"Valider\">";
                    } else {
                        $html .= "erreur, pas de réponse !";
                    }
                } catch (\GuzzleHttp\Exception\RequestException $e) {
                    $html .= "erreur : " . print_r($e, true);
                }
            }
        } else {
            $client = new GuzzleHttp\Client($opt);
            try {
                $response = $client->request('POST', $data['poll']['endpoint'], [
                    'form_params' => [
                        'token' => $data['poll']['token']
                    ]
                ]);
                if ($response->getStatusCode() == 200) {
                    $data = json_decode((string) $response->getBody(), true);
                    $config['cloud_uri'] = $data['server'] . "/";
                    $info = [
                        'id' => $data['loginName'],
                        'code' => $data['appPassword']
                    ];
                    $res = create_rclone_config($config, $info, $user);
                    if ($res === true) {
                        $html .= "<p>Votre connexion à <a href='https://apps.education.fr'>apps.education.fr</a> a bien été activée.<br>
Vous trouverez sur votre bureau un dossier <b>nuages\\nuage.apps</b> vous donnant accès à vos données.";
                    } else {
                        $html .= $res;
                    }
                } elseif ($response->getStatusCode() == 404) {
                    $html .= "Attente de l'authentification sur nuage<br> 
<input type=hidden name=data value=" . serialize($data) . ">
<script type=\"text/javascript\">
    var wait = setTimeout(document.cloud.submit(), 29000)
</script>
<input type=hidden name=nuage value=" . $nuage . ">
<input type=\"submit\" name=\"valider\" value=\"Valider\">";
                } else {
                    $html .= "erreur : " . $response->getStatusCode();
                }
            } catch (\GuzzleHttp\Exception\ConnectException $e) {
                $html .= "erreur : <pre>" . print_r($e, true) . "</pre>";
            }
        }
        $html .= '</form>';
    }
    return $html;
}

function get_seafile_client_code($config, $user, $password)
{
    $info = false;
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('POST', 'api2/auth-token/', [
            'form_params' => [
                'username' => $user . "@" . $config['domain'],
                'password' => $password
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['token']) {
                $info['code'] = $res['token'];
                $info['id'] = $user;
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException $e) {
        return false;
    }
    return $info;
}

function test_rclone_config($config, $login)
{
    $rclone_config = "/home/$login/.config/rclone/rclone.conf";
    exec("sudo stat " . $rclone_config . " 2>&1", $out, $ret);
    if ($ret == 0) {
        $rclone = "rclone lsjson " . $config['cloud_name'] . ":/ --max-depth=1 --config=$rclone_config 2>/dev/null";
        exec(sudo_proxy($config) . $rclone, $out, $ret);
        if ($ret == 0) {
            return true;
        } else {
            return false;
        }
    } else {
        return false;
    }
}

function create_rclone_config($config, $info, $user)
{
    $rclone_config = "/home/" . $user['cn'] . "/.config/rclone/rclone.conf";
    system("sudo mkdir -p /home/" . trim($user['cn']) . "/.config/rclone/");
    system("sudo chown -R " . $user['cn'] . " /home/" . $user['cn'] . "/.config/");

    switch (@$config['cloud_type']) {
        case "seafile":
            $rclone = "rclone config create " . $config['cloud_name'] . " " . $config['cloud_type'] . " vendor=sambaedu url=\"" . cloud_path($config, $user['cn']) . "\" token=" . $info['code'] . " --obscure --config=$rclone_config";
            break;
        default:
            if (@$config['cloud_name'] != "nuage.apps") {
                $rclone = "rclone config create " . $config['cloud_name'] . " " . $config['cloud_type'] . " vendor=sambaedu url=\"" . cloud_path($config, urlencode($user['Id NC'])) . "\" user=\"" . $user['cn'] . "\" pass=" . @$info['code'] . " --obscure --config=$rclone_config";
            } else {
                $rclone = "rclone config create " . $config['cloud_name'] . " " . $config['cloud_type'] . " vendor=sambaedu url=\"" . cloud_path($config, urlencode($info['id'])) . "\" user=\"" . $info['id'] . "\" pass=" . @$info['code'] . " --obscure --config=$rclone_config";
            }
    }
    exec(sudo_proxy($config) . $rclone, $out, $ret);
    if ($ret == 0) {
        exec("sudo chown " . $user['cn'] . " $rclone_config");
        groupaddmember($config, $user['cn'], "Cloud");
        $ret = test_rclone_config($config, $user['cn']);
    } else {
        $ret = false;
    }
    // return "erreur creation rclone : <pre> $rclone pour $login $id " . implode("<br>", $out) . "</pre>";
    return $ret;
}

function configure_user_cloud($config, $login, $password)
{
    $user = search_user($config, $login);
    // $user = nc_users_ids($config, $user)[0];
    $ret = true;
    switch (@$config['cloud_type']) {
        case "seafile":
            // $code = $password;
            $info = get_seafile_client_code($config, $user, $password);
            break;
        case "webdav":
            $info = get_nc_client_code($config, $user['cn'], $password);
            if ($info) {
                $user['Id NC'] = $info['id'];
                $info['storageLocation'] = cloud_path($config, $user['Id NC']);
            }
            break;
        default:
            $info = false;
    }
    if ($info) {
        $ret = create_rclone_config($config, $info, $user);
    }
    return $ret;
}

function get_nc_userdata($config, $user)
{
    $data = false;
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('GET', 'ocs/v2.php/cloud/users/' . $user, [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'query' => [
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                $data = $res['ocs']['data'];
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException $e) {
        return false;
    }
    return $data;
}

/**
 * crée un utilisateur API pour un site et renvoie son id et code api.
 * le résultat est enegistreré dans un fichier pour déploient ultérieur sur les sites.
 * ne pas utiliser pour autre chose que créer un compte api ! et uniquement sur le
 * serv central lors du provisionnement des sites
 *
 * @param unknown $config
 * @param unknown $user
 *            (uai du site)
 * @return boolean
 */
function create_nc_api_user($config, $user)
{
    $name = substr($user, 3);
    if (! file_exists("/etc/sambaedu/cloud/" . $user . ".conf")) {
        $password = create_random_password(20, true, false);
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['cloud_uri'],
            'cookies' => true,
            'headers' => [
                'OCS-APIRequest' => "true"
            ]
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
        try {
            $response = $client->request('POST', 'ocs/v1.php/cloud/users', [
                'auth' => [
                    $config['cloud_api_user'],
                    $config['cloud_api_password']
                ],
                'form_params' => [
                    'userid' => "api-" . $name,
                    'password' => $password,
                    'displayName' => "Api " . $user,
                    'groups' => [
                        'admin'
                    ],
                    'format' => 'json'
                ]
            ]);
            if ($response) {
                $res = json_decode((string) $response->getBody(), true);
                if ($res['ocs']['meta']['status'] == "ok") {
                    $info = get_nc_client_code($config, "api-" . $name, $password);
                    if ($info) {
                        $file = 'cloud_api_user = "api-' . $name . '"
cloud_type = "webdav"
cloud_name = "moncollege95"
cloud_password = "' . $password . '"
cloud_api_password = "' . $info['code'] . '"
cloud_api_id = "' . $info['id'] . '"
cloud_uri = "' . $config['cloud_uri'] . '"
';

                        return ! empty(file_put_contents("/etc/sambaedu/cloud/" . $user . ".conf", $file));
                    }
                }
            }
            return false;
        } catch (\GuzzleHttp\Exception\RequestException $e) {
            return false;
        }
    } else {
        return true;
    }
}

function search_nc_user($config, $user)
{
    if (is_array($user)) {
        $u = $user['cn'];
    } else {
        $u = $user;
    }

    if (! get_nc_userdata($config, $u)) {

        if (! is_array($user)) {
            $ad = search_user($config, $user);
        } else {
            $ad = $user;
        }
        if (! $ad) {
            return false;
        }
        $fullname = $ad['fullname'] ?? $ad['nom'] ?? explode('@', $ad['mail'])[0] ?? explode('@', $ad['userprincipalname'])[0];
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['cloud_uri'],
            'cookies' => true,
            'headers' => [
                'OCS-APIRequest' => "true"
            ]
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
        try {
            $response = $client->request('GET', 'ocs/v2.php/core/autocomplete/get', [
                'auth' => [
                    $config['cloud_api_user'],
                    $config['cloud_api_password']
                ],
                'query' => [
                    'format' => 'json',
                    'search' => $fullname,
                    'limit' => 1
                ]
            ]);
            if ($response) {
                $res = json_decode((string) $response->getBody(), true);
                if ($res['ocs']['meta']['status'] == "ok") {
                    $u = $res['ocs']['data'][0]['id'] ?? false;
                }
            }
        } catch (\GuzzleHttp\Exception\RequestException $e) {
            return false;
        }
    }
    return $u;
}

function nc_users_ids($config, $user = "")
{
    if (empty($user)) {
        if (empty($config['etab_ou'])) {
            $users = search_ad($config, "*", "user", $config['dn']['people']);
        } else {
            $users = search_ad($config, "*", "memberof", $config['etab_ou']);
        }
    } else {
        $users = [
            $user
        ];
    }
    if (! empty($config['cloud_uri'])) {
        $hash = time();
        if (apcu_fetch("ldap_cache_invalid")) {
            apcu_delete(new APCUIterator('#^nc_#'));
            apcu_delete("ldap_cache_invalid");
        } elseif ($ret = apcu_fetch("nc_" . $hash)) {
            return $ret;
        }

        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['cloud_uri'],
            'cookies' => true,
            'headers' => [
                'OCS-APIRequest' => "true"
            ]
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
        $requests = function ($users) use ($client) {
            foreach ($users as $k => $user) {
                yield $k => function () use ($client, $user) {
                    $config = [];
                    $config = get_config($config, true);
                    return $client->getAsync('ocs/v2.php/core/autocomplete/get', [
                        'auth' => [
                            $config['cloud_api_user'],
                            $config['cloud_api_password']
                        ],
                        'query' => [
                            'format' => 'json',
                            'search' => $user['fullname'],
                            'limit' => 1
                        ],
                        'connect_timeout' => 2
                    ]);
                };
            }
        };

        $pool = new GuzzleHttp\Pool($client, $requests($users), [
            'concurrency' => 30,
            'fulfilled' => function (GuzzleHttp\Psr7\Response $response, $index) use (&$users) {
                // this is delivered each successful response
                $res = json_decode((string) $response->getBody(), true);
                if ($res['ocs']['meta']['status'] == "ok") {
                    $users[$index]['Id NC'] = $res['ocs']['data'][0]['id'];
                }
            },
            'rejected' => function (GuzzleException $reason, $index) {
                // this is delivered each failed request
                // echo "erreur : " . $reason->getMessage();
            }
        ]);

        // Initiate the transfers and create a promise
        $promise = $pool->promise();

        // Force the pool of requests to complete.
        $promise->wait();
        apcu_store("nc_" . $hash, $users);
    }
    return $users;
}

function get_rclone_config($config, $user)
{
    $rclone_config = "/home/$user/.config/rclone/rclone.conf";
    $rclone = "rclone config dump --config=$rclone_config";
    exec(sudo_proxy($config) . $rclone, $out, $ret);
    if ($ret == 0) {
        return json_decode(implode(' ', $out), true);
    }
    return false;
}

define('OCS_GROUP', 1);
define('OCS_USER', 0);
define('OCS_PERM_RO', 1);
define('OCS_PERM_RW', 15);

function get_nc_all_shares($config)
{
    $data = false;
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('GET', 'ocs/v2.php/apps/files_sharing/api/v1/shares', [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'query' => [
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                $data = $res['ocs']['data'];
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException $e) {
        return $e;
    }
    return $data;
}

function get_nc_shares($config, $path, $sub = 'false', &$client = NULL)
{
    $data = false;
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    if (! $client) {
        $opt = [
            'base_uri' => $config['cloud_uri'],
            'cookies' => true,
            'headers' => [
                'OCS-APIRequest' => "true"
            ]
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
    }
    try {
        $response = $client->request('GET', 'ocs/v2.php/apps/files_sharing/api/v1/shares', [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'query' => [
                'path' => $path,
                'format' => 'json',
                'subfiles' => $sub
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                $data = $res['ocs']['data'];
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException $e) {
        return false;
    }
    return $data;
}

function create_nc_share($config, $path, $share, &$data = [])
{
    $u = $share['share_with'];
    if ($share['share_type'] == OCS_GROUP || $u = search_nc_user($config, $u)) {

        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['cloud_uri'],
            'cookies' => true,
            'headers' => [
                'OCS-APIRequest' => "true"
            ]
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
        try {
            $response = $client->request('POST', 'ocs/v2.php/apps/files_sharing/api/v1/shares', [
                'auth' => [
                    $config['cloud_api_user'],
                    $config['cloud_api_password']
                ],
                'form_params' => [
                    'path' => $path,
                    'shareType' => $share['share_type'],
                    'shareWith' => $u,
                    'permissions' => $share['permissions'],
                    'format' => 'json'
                ]
            ]);
            if ($response) {
                $res = json_decode((string) $response->getBody(), true);
                $data = $res['ocs']['data'];
                if ($res['ocs']['meta']['status'] == "ok") {
                    return true;
                } else {
                    return false;
                }
            }
        } catch (\GuzzleHttp\Exception\RequestException $e) {
            return false;
        }
        return true;
    } else {
        return false;
    }
}

function modify_nc_share($config, $id, $share, &$data = [])
{
    if ($u = search_nc_user($config, $share['share_with'])) {

        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['cloud_uri'],
            'cookies' => true,
            'headers' => [
                'OCS-APIRequest' => "true"
            ]
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
        try {
            $response = $client->request('PUT', 'ocs/v2.php/apps/files_sharing/api/v1/shares/' . $id, [
                'auth' => [
                    $config['cloud_api_user'],
                    $config['cloud_api_password']
                ],
                'query' => [
                    'permissions' => $share['permissions'],
                    'format' => 'json'
                ]
            ]);
            if ($response) {
                $res = json_decode((string) $response->getBody(), true);
                $data = $res['ocs']['data'];
                if ($res['ocs']['meta']['status'] == "ok") {
                    return true;
                } else {
                    return false;
                }
            }
        } catch (\GuzzleHttp\Exception\RequestException $e) {
            return false;
        }
        return true;
    } else {
        return false;
    }
}

function delete_nc_share($config, $id)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('DELETE', 'ocs/v2.php/apps/files_sharing/api/v1/shares/' . $id, [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'query' => [
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            $data = $res['ocs']['data'];
            if ($res['ocs']['meta']['status'] == "ok") {
                return true;
            } else {
                return false;
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException $e) {
        return false;
    }
    return true;
}

// webdav
function dav_proxy_options($config, &$opt = [])
{
    if (! empty($config['server_proxy']) && ! empty($config['proxy_address']) && ! empty($config['proxy_port'])) {
        $opt['proxy'] = "http://" . $config['proxy_address'] . ":" . $config['proxy_port'];
    }
}

/**
 * liste l'ensemble des dossiers avec leurs partages éventuels
 *
 * fonction destinée à la synchro des dossiers groupes. Pas forcément adaptée à autre chose.
 * retourne un tableau avec les élements compatibles avec la création ou la suppression.
 *
 * @param unknown $config
 * @param string $path
 * @param unknown $client
 * @param number $depth
 * @return \Sabre\HTTP\ClientHttpException|array
 */
function list_nc_files($config, $path = "", &$client = NULL, $depth = 3)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');

    $user = $config['cloud_api_user'];

    if (! $client) {

        $opt = array(
            'baseUri' => cloud_path($config, $user),
            // 'userName' => $user,
            'userName' => $config['cloud_api_user'],
            'password' => $config['cloud_api_password']
        );
        dav_proxy_options($config, $opt);
        $client = new Sabre\DAV\Client($opt);
    }

    try {
        $data = $client->propfind($path, array(
            '{http://owncloud.org/ns}share-types',
            '{http://owncloud.org/ns}owner-display-name',
            '{http://owncloud.org/ns}fileid'
        ), $depth);
        // return $data;
    } catch (Sabre\HTTP\ClientException | Sabre\HTTP\ClientHttpException $e) {
        return $e;
    }
    $shares = get_nc_all_shares($config);
    $list = [];
    foreach ($data as $dav_path => $info) {
        $dav_path = urldecode($dav_path);
        preg_match("!/remote\.php/dav/files/" . $user . "(/?" . $path . ".*)/!", $dav_path, $m);
        $relative_path = $m[1] ?? "";
        if (! empty($relative_path)) {
            $keys = array_keys(array_column($shares, 'path'), $relative_path);
            if (count($keys) > 0) {
                foreach ($keys as $key) {
                    $list[$relative_path][$shares[$key]['id']] = [
                        'type' => $shares[$key]['share_type'],
                        'permissions' => $shares[$key]['permissions'],
                        'share_with' => $shares[$key]['share_with']
                    ];
                }
            } else {
                $list[$relative_path] = [];
            }
        }
    }
    uksort($list, 'path_sort');
    return $list;
}

function create_nc_folder($config, $path, &$client)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');

    if (! $client) {
        $user = $config['cloud_api_user'];

        $opt = array(
            'baseUri' => cloud_path($config, $user),
            'userName' => $user,
            'password' => $config['cloud_api_password']
        );
        dav_proxy_options($config, $opt);
        $client = new Sabre\DAV\Client($opt);
    }

    try {
        $data = $client->request('MKCOL', str_replace(" ", "%20", ltrim($path, "/")));
        if ($data['statusCode'] == '201') {
            return true;
        } else {
            return false;
        }
    } catch (Sabre\HTTP\ClientException | Sabre\HTTP\ClientHttpException $e) {
        // $m = "Erreur :" . print_r($e, true);
        return false;
    }
}

function delete_nc_folder($config, $path)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');

    $user = $config['cloud_api_user'];
    $opt = array(
        'baseUri' => cloud_path($config, $user),
        'userName' => $user,
        'password' => $config['cloud_api_password']
    );
    dav_proxy_options($config, $opt);
    $client = new Sabre\DAV\Client($opt);

    try {
        $data = $client->propfind($path, array(
            '{http://owncloud.org/ns}fileid'
        ), 20);
        $res = ($data['statusCode'] == '201');
        uksort($data, 'path_sort_desc');
        foreach ($data as $dav_path => $info) {
            $m = [];
            $dav_path = urldecode($dav_path);
            preg_match("!/remote\.php/dav/files/" . $user . "/(.*)$!", $dav_path, $m);
            $path = $m[1] ?? "";
            $data = $client->request('DELETE', str_replace(" ", "%20", ltrim($path, "/")));
            $res &= ($data['statusCode'] == '201');
        }
        return $res;
    } catch (Sabre\HTTP\ClientException | Sabre\HTTP\ClientHttpException $e) {
        return false;
    }
}

function move_nc_folder($config, $path, $newpath)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');

    $user = $config['cloud_api_user'];

    $opt = array(
        'baseUri' => cloud_path($config, $user),
        'userName' => $user,
        'password' => $config['cloud_api_password']
    );
    dav_proxy_options($config, $opt);
    $client = new Sabre\DAV\Client($opt);
    try {
        $data = $client->request('MOVE', str_replace(" ", "%20", ltrim($path, "/")), NULL, [
            'Destination' => cloud_path($config, $user) . str_replace(" ", "%20", ltrim($newpath, "/")),
            'Overwrite' => 'T'
        ]);
        if ($data['statusCode'] == '201' || $data['statusCode'] == '204') {
            return true;
        } else {
            return false;
        }
    } catch (Sabre\HTTP\ClientException | Sabre\DAV\Exception $e) {
        return false;
    }
}

function list_nc_echanges($config, $prof)
{
    $echanges = [];
    if (is_array($prof)) {
        $classes = $prof['memberof'];
    } else {
        $classes = search_user($config, $prof)['memberof'];
    }
    foreach ($classes as $classe) {
        $m = array();

        if (preg_match("/CN=Equipe_(.*),OU=equipe.*/i", $classe, $m)) {
            $nom = $m[1];
            if (count(search_ad($config, $nom, "classe")) > 0) {
                $type = "Classe";
            } else {
                $type = "Cours";
            }
            $res = get_nc_shares($config, "/Groupes/@" . $nom . "/_Echange@" . $nom);
            $echanges[$nom] = [
                'id' => 0,
                'type' => "Classe"
            ];
            if (count($res) > 0) {
                foreach ($res as $s) {
                    if ($s['share_type'] == OCS_GROUP && $s['permissions'] == OCS_PERM_RW && $s['share_with'] == $type . "_" . $nom . $config['suffix']) {
                        $echanges[$nom] = [
                            'id' => $s['id'],
                            'type' => $type
                        ];
                        break;
                    }
                }
            }
        }
    }

    return $echanges;
}

function set_nc_echanges($config, &$echanges)
{
    $res = true;
    foreach ($echanges as $nom => $e) {
        if ($e['id'] == 0) {
            $res &= delete_nc_share($config, $e['id']);
        } else {
            $s = [
                'share_type' => OCS_GROUP,
                'share_with' => $e['type'] . "_" . $nom . $config['suffix'],
                'permissions' => OCS_PERM_RW
            ];
            $res &= create_nc_share($config, "/Groupes/@" . $nom . "/_Echange@" . $nom, $s);
        }
        $shares = get_nc_shares($config, "/Groupes/@" . $nom . "/_Echange@" . $nom);
        if (count($shares) > 0) {
            foreach ($shares as $s) {
                if ($s['share_type'] == OCS_GROUP && $s['permissions'] == OCS_PERM_RW && $s['share_with'] == $e['type'] . "_" . $nom . $config['suffix']) {
                    $echanges[$nom] = [
                        'id' => $s['id'],
                        'type' => $e['type']
                    ];
                    $res &= $res;
                    break;
                }
            }
        }
    }
    return $res;
}

/**
 * Bascule un partage échange entre l'état partage et non partagé
 *
 * @param unknown $config
 * @param unknown $nom
 * @param unknown $echanges
 *            liste des Echanges en cache
 * @return boolean
 */
function toggle_nc_echange($config, $nom, &$echanges)
{
    $res = false;
    $e = $echanges[$nom];
    if ($e['id'] != 0) {
        $res = delete_nc_share($config, $e['id']);
        $echanges[$nom] = [
            'id' => 0,
            'type' => $e['type']
        ];
    } else {
        $s = [
            'share_type' => OCS_GROUP,
            'share_with' => $e['type'] . "_" . $nom . $config['suffix'],
            'permissions' => OCS_PERM_RW
        ];
        if ($res = create_nc_share($config, "/Groupes/@" . $nom . "/_Echange@" . $nom, $s)) {
            $res = get_nc_shares($config, "/Groupes/@" . $nom . "/_Echange@" . $nom);
            if (count($res) > 0) {
                foreach ($res as $s) {
                    if ($s['share_type'] == OCS_GROUP && $s['permissions'] == OCS_PERM_RW && $s['share_with'] == $e['type'] . "_" . $nom . $config['suffix']) {
                        $echanges[$nom] = [
                            'id' => $s['id'],
                            'type' => $e['type']
                        ];
                        break;
                    }
                }
            }
        }
    }
    return $res;
}

function path_sort($a, $b)
{
    return count(explode('/', $a)) - count(explode('/', $b));
}

function path_sort_desc($a, $b)
{
    return count(explode('/', $b)) - count(explode('/', $a));
}

/**
 * calcule le tableau des différences entre deux états des partages a et b
 * génère un tableau de la forme ['path' => ['id'=> [share]]]
 * si 'path'=> [], on déplace ou efface le dossier
 * si 'id' = 0, on crée le partage
 * si 'id' != 0 :
 * si share = [] on supprime le partage
 * sinon on le modifie
 *
 * Ne modifie pas le partage des dossiers Echange, mais les crée ou supprime si besoin.
 *
 * attention l'algo ne fonctionne que si il n'y a qu'un seul partage par dossier !!!
 *
 * @param array $aa
 *            : état souhaité (appartenance aux groupes pour l'ajout)
 * @param array $bb
 *            : état à soustraire (Arbo NC pour l'ajout)
 * @param boolean $del
 *            : si true on prend en compte les dossiers dans archives pour la suppression
 * @return []
 */
function diff_shares($config, $aa, $bb, $del = false)
{
    $diff = [];
    // recherche des dossiers à restaurer
    foreach ($bb as $pb => $b) {
        $m = [];
        // On regarde les dossiers dans archives uniquement
        if (preg_match("|^/Archives/([^/]*)@([^/]*)@Archives$|", $pb, $m)) {
            $prb = $pb;
            $pb = "/Groupes/@" . $m[2] . "/" . $m[1] . "@" . $m[2];
            foreach ($aa as $pa => $a) {
                if ($pa == $pb) {
                    $diff[$prb] = $b;
                    // on ne tient pas compte du dossier pouvant exister
                    unset($bb[$pb]);
                    unset($aa[$pb]);
                }
            }
        }
    }
    foreach ($aa as $pa => $a) {
        $m = false;
        // pas de prise en compte des dossiers déjà dans Archives pour la suppression
        if (! preg_match("|/Archives/|", $pa) || $del) {
            foreach ($bb as $pb => $b) {
                if ($pa == $pb) {
                    if (preg_match("|/_Echange@|", $pa)) {
                        $m = true;
                        break;
                    } else {
                        if (count($a) > 0) {
                            foreach ($a as $ida => $sa) {
                                if (count($b) > 0) {
                                    foreach ($b as $idb => $sb) {
                                        if ($ida == $idb) {
                                            // même share id, on modifie
                                            $s = array_diff($sa, $sb);
                                            if (count($s) > 0) {
                                                $diff[$pa][$ida] = $sa;
                                            }
                                        } elseif ($ida == 0) {
                                            if ($sa['share_with'] != $sb['share_with']) {
                                                if ($u = search_nc_user($config, $sa['share_with'])) {
                                                    if ($u != $sb['share_with']) {
                                                        $sa['share_with'] = $u;
                                                        // plus de partage, ou changemnt de proprio on efface $idb
                                                        $diff[$pa][$idb] = [];
                                                        // on cree $ida
                                                        $diff[$pa][$ida] = $sa;
                                                    }
                                                }
                                            } else {
                                                // on modifie $idb
                                                $s = array_diff($sa, $sb);
                                                if (count($s) > 0) {
                                                    $diff[$pa][$idb] = $sa;
                                                }
                                            }
                                        }
                                    }
                                } else {
                                    // on cree $ida
                                    $diff[$pa][$ida] = $sa;
                                }
                            }
                        }
                        $m = true;
                        break;
                    }
                }
            }
            if (! $m)
                $diff[$pa] = $a;
        }
    }
    uksort($diff, 'path_sort');
    return $diff;
}

/**
 * mise à jour de l'arborescence du partage pour un groupe.
 * crée les dossiers manquants :
 * Groupes/@groupe -> partagé avec Equipe_groupe
 * Groupes/@groupe/eleve@groupe -> partagé avec eleve
 * déplace les dossiers en trop dans Archive/eleve@groupe@archive -> partagé avec eleve
 * restaure les dossiers Archive/eleve@groupe@archive -> Groupes/@groupe/eleve@groupe Si besoin
 *
 * La mise a jour de l'arborescence côté utilisateur est faite par un script de logon sur le montage rclone
 *
 * fonction agissant sur un cache
 *
 * @param array $config
 * @param string $html
 * @param bool $delete
 * @return boolean
 */
function update_nc_partages($config, &$html, $delete)
{
    $end = false;
    $time = time();
    $partages = apcu_fetch('partages');
    if (! $partages) {
        $liste = list_nc_files($config, "", $client, 3);
        // foreach ($liste as $p => $val){
        // if (preg_match("/\/Archives\/CIM1/", $p)){
        // $l[$p] = $val;
        // }
        // }
        // $html .= "<pre>" . print_r($l, true) . "</pre>\n";
        // return true;
        $groupes = list_partages_groups($config);
        if ($groupes === false) {
            return false;
        }
        // foreach ($groupes as $p => $val){
        // if (preg_match("/CIM2/", $p)){
        // $l[$p] = $val;
        // }
        // }
        // $html .= "<pre>" . print_r($l, true) . "</pre>\n";
        $add = diff_shares($config, $groupes, $liste);
        $remove = diff_shares($config, $liste, $groupes, $delete);
        $html .= count($remove) . " répertoires à supprimer<br>";
        $html .= count($add) . " répertoires à créer<br>";
        // $html .= "<pre>" . print_r($remove, true) . "</pre>\n";
        // $html .= "<pre>" . print_r($add, true) . "</pre>\n";
        // return true;
    } else {
        $add = $partages['add'];
        $remove = $partages['remove'];
        $html .= count($remove) . " répertoires restants à supprimer<br>";
        $html .= count($add) . " répertoires restants à créer<br>";
    }
    while (count($remove) > 0 && time() < $time + 25) {
        $p = key($remove);
        array_shift($remove);
        $m = [];
        unset($n);
        preg_match("!/Groupes/@([^/]*)/?([^@]*)@?(.*)!", $p, $m);
        if (! empty($m[2])) {
            switch ($m[2]) {
                case "Prof":
                    $n = "/Groupes/@" . $m[1] . "/_Equipe@" . $m[3];
                    $html .= "deplacement de $p vers  $n <br>\n";
                    $res = move_nc_folder($config, $p, $n);
                    break;
                case "Echange":
                case "Travail":
                case "Equipe":
                    $n = "/Groupes/@" . $m[1] . "/_" . $m[2] . "@" . $m[3];
                    $html .= "deplacement de $p vers  $n <br>\n";
                    $res = move_nc_folder($config, $p, $n);
                    break;
                default:
                    $html .= "deplacement de $p vers  /Archives/" . $m[2] . "@" . $m[3] . "@Archives<br>\n";
                    $res = move_nc_folder($config, $p, "/Archives/" . $m[2] . "@" . $m[3] . "@Archives");
            }
            if ($res) {
                $html .= "OK<br>\n";
                if (isset($n)) {
                    unset($add[$n]);
                }
            } else {
                $html .= " ERREUR  $res<br>\n";
            }
        } elseif (! empty($m[1])) {
            $html .= "effacement de $p <br>\n";
            $res = delete_nc_folder($config, $p);
            if ($res) {
                $html .= "OK<br>\n";
            } else {
                $html .= " ERREUR  $res<br>\n";
            }
        }
        if ($delete) {
            $m = [];
            preg_match("!/Archives/(.*)!", $p, $m);
            if (! empty($m[1])) {
                $html .= "effacement de $p <br>\n";
                $res = delete_nc_folder($config, $p);
                if ($res) {
                    $html .= "OK<br>\n";
                } else {
                    $html .= " ERREUR  $res<br>\n";
                }
            }
        }
    }
    $client = NULL;
    while (count($add) > 0 && time() < $time + 25) {
        $res = false;
        $p = array_key_first($add);
        $share = array_shift($add);
        $m = [];
        if (preg_match("|^/Archives/([^/]*)@([^/]*)@Archives$|", $p, $m)) {
            $newp = "/Groupes/@" . $m[2] . "/" . $m[1] . "@" . $m[2];
            // $res1 = delete_nc_folder($config, $newp);
            $res1 = move_nc_folder($config, $p, $newp);
            if ($res1) {
                $html .= "restauration de $newp OK, <br>\n";
            } else {
                $html .= "restauration de $newp ERREUR<br>\n";
            }
        } else {
            $res1 = create_nc_folder($config, $p, $client);
            if ($res1) {
                $msg = "creation de $p, \n";
            } else {
                $msg = "creation de $p ERREUR  $res1, \n";
            }
            if (is_array($share)) {
                $res = true;
                foreach ($share as $id => $s) {
                    if ($id != 0) {
                        if (count($s) == 0) {
                            $res &= delete_nc_share($config, $id);
                        } else {
                            $res &= modify_nc_share($config, $id, $s);
                        }
                    } else {
                        $res &= create_nc_share($config, $p, $s);
                    }
                    // $html .= "<pre>" . print_r($data, true) . "</pre>\n";
                }
            }
            if ($res) {
                $html .= "partage de $p OK<br>\n";
            } else {
                if (! $res1) {
                    $html .= $msg;
                }
                $html .= "partage de $p ERREUR<br>\n";
                // $client2 = NULL;
                // $html .= "<pre>" . print_r(get_nc_shares($config, $p, $client2), true) . "</pre>\n";
            }
        }
    }
    $partages = [
        'add' => $add,
        'remove' => $remove
    ];
    apcu_store('partages', $partages, 1000);
    if ((count($add) + count($remove)) == 0) {
        $end = true;
    }
    if ($end) {
        apcu_delete('partages');
    }
    return $end;
}

/** met a jour le dossier perso de l'utilisateur sur NC
 * 
 * Crée l'arborescence côté utilisateur, en renommant si besoin les liens de partages définis par le compte api-etab
 * Aucune donnée n'est modifiée par cette fonction.
 * 
 * les modifications sont faites via rclone côté serveur, avec les identifiants de l'utilisateur. Cela suppose que son compte soit activé.
 * 
 * @param unknown $config
 * @param unknown $info
 * @return boolean
 */
function update_user_dir($config, $info)
{
    $file = "# script pour " . $info['user']['cn'] . "\n";
    $rclone_config = "/home/" . $info['user']['cn'] . "/.config/rclone/rclone.conf";
    exec("sudo stat " . $rclone_config . " 2>&1", $out, $ret);
    if ($ret == 0) {
        $rclone = "rclone lsjson " . $config['cloud_name'] . ":/ --max-depth=2 --config=$rclone_config --no-mimetype --no-modtime --retries-sleep 10s 2>/dev/null";
        $file .= $rclone . "\n";
        unset($out);
        exec(sudo_proxy($config) . $rclone, $out, $ret);
        if ($ret == 0) {
            $out = json_decode(implode("\n", $out), true);
            $file .= print_r($out, true) . "\n";
            if (in_array('Profs', $info['list_u'])) {
                if (! in_array("Mes groupes", array_column($out, 'Path'))) {
                    $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":/Mes groupes\"  --config=" . $rclone_config . " 2>/dev/null";
                    $file .= $rclone . "\n";
                    exec(sudo_proxy($config) . $rclone, $out1, $ret);
                }
                foreach ($info['list_u'] as $groupe) {
                    if (preg_match("/Equipe_(.*)/", $groupe, $m)) {
                        if (in_array("@" . $m[1], array_column($out, 'Path'))) {
                            if (in_array("Mes groupes/" . $m[1], array_column($out, 'Path'))) {
                                $rclone = "rclone move \"" . $config['cloud_name'] . ":Mes groupes/" . $m[1] . "\" \"" . $config['cloud_name'] . ":@" . $m[1] . "_X\" --config=" . $rclone_config . " 2>/dev/null";
                                exec(sudo_proxy($config) . $rclone, $out1, $ret);
                            }
                            $rclone = "rclone move \"" . $config['cloud_name'] . ":@" . $m[1] . "\" \"" . $config['cloud_name'] . ":Mes groupes/" . $m[1] . "\"  --config=" . $rclone_config . " 2>/dev/null";
                            exec(sudo_proxy($config) . $rclone, $out1, $ret);
                        }
                    }
                }
            } else {
                if (! in_array("Mes travaux", array_column($out, 'Path'))) {
                    $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":Mes travaux\"  --config=" . $rclone_config . " 2>/dev/null";
                    $file .= $rclone . "\n";

                    exec(sudo_proxy($config) . $rclone, $out1, $ret);
                }
                if (! in_array("Travail demandé", array_column($out, 'Path'))) {
                    $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":Travail demandé\"  --config=" . $rclone_config . " 2>/dev/null";
                    $file .= $rclone . "\n";
                    exec(sudo_proxy($config) . $rclone, $out1, $ret);
                }
                if (! in_array("Echange", array_column($out, 'Path'))) {
                    $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":Echange\"  --config=" . $rclone_config . " 2>/dev/null";
                    $file .= $rclone . "\n";
                    exec(sudo_proxy($config) . $rclone, $out1, $ret);
                }
                $file .= print_r($info, true) . "\n";
                foreach ($info['list_u'] as $groupe) {
                    if (preg_match("/(Classe_|Cours_)(.*)/", $groupe, $m)) {
                        $file .= $m[2] . " : $groupe\n";
                        if (in_array($info['user']['cn'] . "@" . $m[2], array_column($out, 'Path'))) {
                            if (in_array("Mes travaux/" . $m[2], array_column($out, 'Path'))) {
                                $rclone = "rclone move \"" . $config['cloud_name'] . ":Mes travaux/" . $m[2] . "\" \"" . $config['cloud_name'] . ":" . $info['user']['cn'] . "@" . $m[2] . "_U\" --config=" . $rclone_config . " 2>/dev/null";
                                $file .= $rclone . "\n";
                                exec(sudo_proxy($config) . $rclone, $out1, $ret);
                            }
                            $rclone = "rclone move \"" . $config['cloud_name'] . ":" . $info['user']['cn'] . "@" . $m[2] . "\" \"" . $config['cloud_name'] . ":Mes travaux/" . $m[2] . "\"  --config=" . $rclone_config . " 2>/dev/null";
                            $file .= $rclone . "\n";
                            exec(sudo_proxy($config) . $rclone, $out1, $ret);
                        }
                        if (in_array("_Travail@" . $m[2], array_column($out, 'Path'))) {
                            if (in_array("Travail demandé/" . $m[2], array_column($out, 'Path'))) {
                                $rclone = "rclone move \"" . $config['cloud_name'] . ":Travail demandé/" . $m[2] . "\" \"" . $config['cloud_name'] . ":" . $info['user']['cn'] . "@" . $m[2] . "_T\" --config=" . $rclone_config . " 2>/dev/null";
                                $file .= $rclone . "\n";
                                exec(sudo_proxy($config) . $rclone, $out1, $ret);
                            }
                            $rclone = "rclone move \"" . $config['cloud_name'] . ":_Travail@" . $m[2] . "\" \"" . $config['cloud_name'] . ":Travail demandé/" . $m[2] . "\"  --config=" . $rclone_config . " 2>/dev/null";
                            $file .= $rclone . "\n";
                            exec(sudo_proxy($config) . $rclone, $out1, $ret);
                        }
                        if (in_array("Travail@" . $m[2], array_column($out, 'Path'))) {
                            if (in_array("Travail demandé/" . $m[2], array_column($out, 'Path'))) {
                                $rclone = "rclone move \"" . $config['cloud_name'] . ":Travail demandé/" . $m[2] . "\" \"" . $config['cloud_name'] . ":" . $info['user']['cn'] . "@" . $m[2] . "_T\" --config=" . $rclone_config . " 2>/dev/null";
                                $file .= $rclone . "\n";
                                exec(sudo_proxy($config) . $rclone, $out1, $ret);
                            }
                            $rclone = "rclone move \"" . $config['cloud_name'] . ":Travail@" . $m[2] . "\" \"" . $config['cloud_name'] . ":Travail demandé/" . $m[2] . "\"  --config=" . $rclone_config . " 2>/dev/null";
                            $file .= $rclone . "\n";
                            exec(sudo_proxy($config) . $rclone, $out1, $ret);
                        }
                        if (in_array("_Echange@" . $m[2], array_column($out, 'Path'))) {
                            if (in_array("Echange/" . $m[2], array_column($out, 'Path'))) {
                                $rclone = "rclone move \"" . $config['cloud_name'] . ":Echange/" . $m[2] . "\" \"" . $config['cloud_name'] . ":" . $info['user']['cn'] . "@" . $m[2] . "_E\" --config=" . $rclone_config . " 2>/dev/null";
                                $file .= $rclone . "\n";
                                exec(sudo_proxy($config) . $rclone, $out1, $ret);
                            }
                            $rclone = "rclone move \"" . $config['cloud_name'] . ":_Echange@" . $m[2] . "\" \"" . $config['cloud_name'] . ":Echange/" . $m[2] . "\"  --config=" . $rclone_config . " 2>/dev/null";
                            $file .= $rclone . "\n";
                            exec(sudo_proxy($config) . $rclone, $out1, $ret);
                        }
                        if (in_array("Echange@" . $m[2], array_column($out, 'Path'))) {
                            if (in_array("Echange/" . $m[2], array_column($out, 'Path'))) {
                                $rclone = "rclone move \"" . $config['cloud_name'] . ":Echange/" . $m[2] . "\" \"" . $config['cloud_name'] . ":" . $info['user']['cn'] . "@" . $m[2] . "_E\" --config=" . $rclone_config . " 2>/dev/null";
                                $file .= $rclone . "\n";
                                exec(sudo_proxy($config) . $rclone, $out1, $ret);
                            }
                            $rclone = "rclone move \"" . $config['cloud_name'] . ":Echange@" . $m[2] . "\" \"" . $config['cloud_name'] . ":Echange/" . $m[2] . "\"  --config=" . $rclone_config . " 2>/dev/null";
                            $file .= $rclone . "\n";
                            exec(sudo_proxy($config) . $rclone, $out1, $ret);
                        }
                    }
                }
            }
            // file_put_contents("/tmp/rclone.log", $file);
            return true;
        } else {
            return false;
        }
    } else {
        return false;
    }
}

/**
 * génération du script qui va déplacer les dossiers partagés de l'élève dans son arborescence.
 * Lancé par les scripts applications, une fois le partage monté par rclone.
 * Nextcloud place les dossiers partagés à la racine, on les déplace pour que cela soit compréhensible.
 * Pour éviter les collisions avec des fichiers de l'utilisateur, ils sont nommés :
 * Groupes/@groupe/Travail@groupe
 * /x@groupe
 * /...@groupe
 * En cas de suppression des dossiers par l'utilisateur, ils sont remis en place au logon suivant (ce ne sont que des liens vers les données réelles du NC)
 *
 * l'élève verra par exemple :
 * Mes travaux\1G1
 * \PhysSpe
 * Travail demandé\1G1
 * \PhysSpe
 * Echanges\1G1
 * \PhysSpe
 *
 *
 * le prof verra par exemple :
 * Mes groupes\1G1\...
 * \TSTI1\Profs@TSTI1
 * \Travail@TSTI1
 * \Echange@TSTI1
 * \x@TSTI1
 * \y@TSTI1
 * \NSI\...
 * \I2D\...
 *
 * Il n'est pas possible pour les profs de renommer les sous dossiers individuels car cela poserait des conflits pour les élèves. seuls les liens de partage peuvent être renommés.
 *
 * Les droits et la visibilité sont définis par l'API, et donc si le dossier échange par exemple est non partagé il ne sera pas vu et créé.
 *
 * Le montage est unique pour les profs.
 *
 * Pour les élèves un montage séparé est fait pour chaque sous-dossier afin de bien gérer les droits des partages
 *
 * @param unknown $config
 * @param unknown $id
 * @return string
 */
function genere_script_partage_utilisateur($config, $id)
{
    $script = "";
    if ($info = apcu_fetch("apps." . $id)) {
        $ret = 0;
        $out = [];
        $rclone_config = "/home/" . $info['user']['cn'] . "/.config/rclone/rclone.conf";
        exec("sudo stat " . $rclone_config . " 2>&1", $out, $ret);
        if ($ret == 0) {
            $rclone = "rclone listremotes --config=$rclone_config 2>/dev/null";
            $remotes = [];
            exec(sudo_proxy($config) . $rclone, $remotes, $ret);
            $i = 0;
            foreach ($remotes as $remote) {
                if ($info['os'] == "windows") {
                    $path = '%userprofile%\\nuages';
                    $script .= "if not exist " . $path . " (md " . $path . ")\r\n";
                }
                $remote = rtrim($remote, ":");
                if (! empty($config['cloud_name']) && $remote == $config['cloud_name']) {
                    if ($ret = update_user_dir($config, $info)) {
                        if (in_array('Profs', $info['list_u'])) {
                            //
                            // profs
                            //
                            $script .= rclone_mount($config, $info, $remote, "", $i);
                        } else {
                            //
                            // eleves
                            //
                            if ($info['os'] == "windows") {
                                $path = '%userprofile%\\nuages\\' . $config['cloud_name'];
                                $script .= "md " . $path . "\r\n";
                            } else {
                                // $script .= "mkdir -p /home/\$USER/nuages/" . $config['cloud_name'] . "\n";
                            }
                            $script .= rclone_mount($config, $info, $remote, "Mes travaux", $i);
                            $script .= rclone_mount($config, $info, $remote, "Echange", $i);
                            $script .= rclone_mount($config, $info, $remote, "Travail demandé", $i, "--read-only");
                        }
                    }
                } else {
                    $script .= rclone_mount($config, $info, $remote, "", $i);
                }
            }
        }
    }
    return $script;
}

function rclone_proxy($config, $info, $i)
{
    if ($info['os'] == "windows") {
        if ($config['proxy_type'] != "aucun") {
            $proxy = "echo set http_proxy=http://" . $config['proxy_address'] . ":" . $config['proxy_port'] . ">%temp%/rclone-$i.cmd\r\n";
            $proxy .= "echo set https_proxy=http://" . $config['proxy_address'] . ":" . $config['proxy_port'] . ">>%temp%/rclone-$i.cmd\r\n";
        } else {
            $proxy = "echo rem pas de proxy>%temp%/rclone-$i.cmd\r\n";
        }
    } else {
        if ($config['proxy_type'] != "aucun") {
            $proxy = "   Environment=http_proxy=http://" . $config['proxy_address'] . ":" . $config['proxy_port'] . "\n";
            $proxy .= "   Environment=https_proxy=http://" . $config['proxy_address'] . ":" . $config['proxy_port'] . "\n";
        } else {
            $proxy = "    # Pas de proxy\n";
        }
    }
    return $proxy;
}

function rclone_mount($config, $info, $remote, $sub, &$i, $args = "")
{
    if ($info['os'] == "windows") {
        if (! empty($sub)) {
            $path = "%userprofile%\\nuages\\" . $remote . "\\" . $sub;
        } else {
            $path = "%userprofile%\\nuages\\" . $remote;
        }
        $script = "if exist \"" . $path . "\" (rd /S /Q \"" . $path . "\")\r\n";
        $script .= rclone_proxy($config, $info, $i);
        $script .= "echo \"%programfiles%\\rclone\\rclone.exe\" mount \"" . $remote . ":" . $sub . "\" \"" . $path . "\" " . $args . " --config \\\\%SE4FS%\\users\\%username%\\.config\\rclone\\rclone.conf --vfs-cache-mode full --cache-dir %temp%\\rclone --no-console>>%temp%\\rclone-$i.cmd\r\n";
        $script .= "start /min %temp%\\rclone-$i.cmd\r\n";
        // $script .= "Set _seconds=0\r
        // :waitloop$i\r
        // Set /a \"_seconds=_seconds+5\">nul\r
        // PING -n 1 127.0.0.1>nul\r
        // if exist \"" . $path . "\" goto nextstep$i\r
        // if %_seconds%==30 goto nextstep$i\r
        // goto waitloop$i
        // :nextstep$i\r
        // ";
        // $script .= "if not exist \"" . $path . "\" (exit 1)\r\n";
        $script = iconv("UTF-8", "CP850", $script);
        $script .= iconv("UTF-8", "Windows-1252", "powershell.exe -ExecutionPolicy Bypass -NoLogo -NonInteractive -NoProfile -Command \"\$o = new-object -com shell.application; \$u=\$o.Namespace('shell:::{679F85CB-0220-4080-B29B-5540CC05AAB6}').Items() | Where-Object { \$_.Name -EQ '" . $remote . "' }; if (\$u -ne \$null) {\$u.InvokeVerb('unpinfromhome')}\"\r\n");
        $script .= iconv("UTF-8", "Windows-1252", "powershell.exe -ExecutionPolicy Bypass -NoLogo -NonInteractive -NoProfile -Command \"\$o = new-object -com shell.application; \$o.Namespace('" . $path . "').Self.InvokeVerb('pintohome')\"\r\n");
        $script .= "\r\n";
    } else {
        if (! empty($sub)) {
            $path = "%h/nuages/" . $remote . "/" . $sub;
        } else {
            $path = "%h/nuages/" . $remote;
        }
        $script = "mkdir -p \"~/nuages/" . $remote . "/" . $sub . "\"\n";
        $script .= "echo \"[Unit]
	Description=$path (rclone)
	AssertPathIsDirectory='" . $path . "'
	# Make sure we have network enabled
	After=network.target
	    
[Service]
	Type=simple
";
        $script .= rclone_proxy($config, $info, $i);
        $script .= "    ExecStart=/usr/bin/rclone mount '" . $remote . ":" . $sub . "' '" . $path . "' --vfs-cache-mode full " . $args . "
            
	# Perform lazy unmount
	ExecStop=/usr/bin/fusermount -zu '" . $path . "'
	
	# Restart the service whenever rclone exists with non-zero exit code
	Restart=on-failure
	RestartSec=15
	
[Install]
	# Autostart after reboot
	WantedBy=default.target\" > ~/.config/systemd/user/rclone-$i.service
	
systemctl --user daemon-reload
systemctl --user enable --now rclone-$i.service\n";
    }
    $i ++;
    return $script;
}

/**
 * construit un tableau avec les utilisateurs classés par catégorie en classe
 * pour l'affichage des listes de selection
 *
 * @param array $config
 * @param string $user
 * @return array|bool
 */
function list_partages_groups($config, $user = "*")
{
    $shares = [
        '/Groupes' => [],
        '/Archives' => []
    ];
    $config['etab_ou'] = $config['etab_ou'] ?? "0";
    // $users = search_ad($config, $user, "user", $config['dn']['people']);
    if ($users = nc_users_ids($config)) {
        foreach ($users as $user) {
            if (isset($user['memberof']) && isset($user['Id NC'])) {
                $u = $user['Id NC'];
                foreach ($user['memberof'] as $groupe) {
                    $m = array();
                    if (preg_match("/CN=(Classe|Equipe|Cours)_(.*),OU=(Classes|Equipes|Cours).*/i", $groupe, $m) && ldap_dn2uai($groupe) == $config['etab_ou']) {
                        if (! isset($c[$m[2]])) {
                            $shares['/Groupes/@' . $m[2]] = [
                                [
                                    'share_type' => OCS_GROUP,
                                    'permissions' => OCS_PERM_RW,
                                    'share_with' => "Equipe_" . $m[2] . $config['suffix']
                                ]
                            ];
                            $shares['/Groupes/@' . $m[2] . '/_Travail@' . $m[2]] = [
                                [
                                    'share_type' => OCS_GROUP,
                                    'permissions' => OCS_PERM_RO,
                                    'share_with' => $m[1] . "_" . $m[2] . $config['suffix']
                                ]
                            ];
                            $shares['/Groupes/@' . $m[2] . '/_Equipe@' . $m[2]] = [];
                            $shares['/Groupes/@' . $m[2] . '/_Echange@' . $m[2]] = [ // [
                                                                                      // 'share_type' => OCS_GROUP,
                                                                                      // 'permissions' => OCS_PERM_RW,
                                                                                      // 'share_with' => $m[1] . "_" . $m[2] . $config['suffix']
                                                                                      // ]
                            ];
                            $c[$m[2]] = true;
                        }

                        if ($m[1] == "Classe" || $m[1] == "Cours") {
                            $shares['/Groupes/@' . $m[2] . '/' . $user['cn'] . "@" . $m[2]] = [
                                [
                                    'share_type' => OCS_USER,
                                    'permissions' => OCS_PERM_RW,
                                    'share_with' => $u
                                ]
                            ];
                        }
                    }
                }
            }
        }
        uksort($shares, 'path_sort');
        return $shares;
    } else {
        return false;
    }
}

function form_partages_next($type = "commit")
{
    $types = array(
        "force" => "Forcer le rafraichissement",
        "clean" => "Nettoyer les partages",
        "commit" => "Rafraîchir les partages"
    );
    $html = "Selectionner l'étape suivante :  <SELECT NAME=\"action\" >"; // onchange='this.form.submit()'>";
    foreach ($types as $t => $txt) {
        $html .= "<option value=\"$t\"";
        if ($type == $t) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$txt</option>";
    }
    $html .= "<input type=\"submit\" value=\"Valider\">";
    $html .= "</SELECT><br>";
    return $html;
}

/**
 * copie les données d'un utilisateur depuis son partage Samba vers le Cloud
 *
 * @param unknown $config
 * @param unknown $user
 * @param unknown $data
 */
function migrate_to_nc($config, $user, $data, $dest = "")
{
    switch ($data) {
        case 'etab':
            $data = $config['cloud_name'];
        case 'nuage.apps':
            $path = "/home/" . $user['cn'] . "/Docs";
            $dest = $data . ":/Documents";
            $command = "rclone copy " . $path . " " . $dest . " --config=/home/" . $user['cn'] . "/.config/rclone/rclone.conf\n";
            // $path = "/home/" . $user['cn'] . "/photos";
            // $dest = $m[1] . ":/Photos";
            // $command = "rclone copy " . $path . " " . $dest . " -config=/home/" . $user['cn'] . "/.config/rclone.conf";
            break;
        case 'classe':
            $classe = list_classes($config, $user)[0];
            $path = "/var/www/sambaedu/Classes/Classe_" . $classe . "/" . inverse_login($user['cn']);
            $dest = $config['cloud_name'] . ":/Mes travaux/" . $classe;
            $command = "rclone copy " . $path . " " . $dest . " --config=/home/" . $user['cn'] . "/.config/rclone/rclone.conf";
            break;
        default:
            $path = $data;
            $command = "rclone copy " . $path . " " . $dest . " --config=/home/" . $user['cn'] . "/.config/rclone/rclone.conf";
    }
    batch_command($command);
    batch_write();
}

function form_nc_migrate($config, $user, $cloud)
{
    $cl = 0;
    $html = "";
    foreach ($user['memberof'] as $g) {
        $cl += preg_match("/cn=Cloud_.*/i", $g);
    }
    if (! empty($_POST['migrate'])) {
        if (groupaddmember($config, $user['cn'], "Cloud_" . $_POST['migrate'])) {
            migrate_to_nc($config, $user, $_POST['migrate']);
            $html .= "Vos données personnelles sont en train d'être transférées vers le serveur " . $_POST['migrate'] . "<br>";
        } else {
            $html .= "Erreur, transfert vers " . $_POST['migrate'] . " impossible.<br>";
        }
    }
    if (! empty($cloud) && $cl == 0) {
        $html .= '<center>Choisir le serveur vers lequel vous voulez migrer vos données personnelles<br>
Attention cette migration aura pour effet de copier tous les fichiers présents dans Mes Documents vers nuage.apps. Le Dossier Mes Documents pointera ensuite vers nuage.apps.
En cas de coupure de la connexion internet, vos documents seront inaccessibles !<br>
Cette opération est définitive.<br>
<form name="cloud-migrate" action="" method="post">';
        foreach ($cloud as $c) {
            $html .= '<input type=submit name="migrate" value="' . $c . '">';
        }
        $html .= '</form></center>';
    }
    return $html;
}
?>
